@@ -28,14 +28,9 @@
 #endif
 
 #define __GOA_BACKEND_INSIDE_GOA_BACKEND_H__
+#include <goabackend/goabackendenumtypes.h>
 #include <goabackend/goabackendtypes.h>
-#include <goabackend/goalogging.h>
-#include <goabackend/goaeditablelabel.h>
 #include <goabackend/goaprovider.h>
-#include <goabackend/goagoogleprovider.h>
-#include <goabackend/goafacebookprovider.h>
-#include <goabackend/goayahooprovider.h>
-#include <goabackend/goatwitterprovider.h>
 #undef __GOA_BACKEND_INSIDE_GOA_BACKEND_H__
 
 #endif /* __GOA_BACKEND_H__ */
