@@ -275,7 +275,7 @@ get_identity_sync (GoaOAuthProvider  *provider,
 static gboolean
 is_deny_node (GoaOAuthProvider *provider, WebKitDOMNode *node)
 {
-  WebKitDOMHTMLElement *element;
+  WebKitDOMElement *element;
   gboolean ret;
   gchar *id;
 
@@ -285,8 +285,8 @@ is_deny_node (GoaOAuthProvider *provider, WebKitDOMNode *node)
   if (!WEBKIT_DOM_IS_HTML_ANCHOR_ELEMENT (node))
     goto out;
 
-  element = WEBKIT_DOM_HTML_ELEMENT (node);
-  id = webkit_dom_html_element_get_id (element);
+  element = WEBKIT_DOM_ELEMENT (node);
+  id = webkit_dom_element_get_id (element);
   if (g_strcmp0 (id, "auth-disallow") != 0)
     goto out;
 
@@ -402,7 +402,6 @@ build_object (GoaProvider         *provider,
 static gboolean
 get_use_external_browser (GoaOAuthProvider *provider)
 {
-  /* For some reason this only works in a browser - bad callback URL? TODO: investigate */
   return FALSE;
 }
 
@@ -413,19 +412,17 @@ show_account (GoaProvider         *provider,
               GoaClient           *client,
               GoaObject           *object,
               GtkBox              *vbox,
-              GtkGrid             *left,
-              GtkGrid             *right)
+              GtkGrid             *grid,
+              G_GNUC_UNUSED GtkGrid *dummy)
 {
-  /* Chain up */
-  GOA_PROVIDER_CLASS (goa_flickr_provider_parent_class)->show_account (provider,
-                                                                       client,
-                                                                       object,
-                                                                       vbox,
-                                                                       left,
-                                                                       right);
+  gint row;
 
-  goa_util_add_row_switch_from_keyfile_with_blurb (left, right, object,
-                                                   NULL,
+  row = 0;
+
+  goa_util_add_account_info (grid, row++, object);
+
+  goa_util_add_row_switch_from_keyfile_with_blurb (grid, row++, object,
+                                                   _("Use for"),
                                                    "photos-disabled",
                                                    _("_Photos"));
 }
