@@ -27,7 +27,9 @@
 #ifndef __GOA_HTTP_CLIENT_H__
 #define __GOA_HTTP_CLIENT_H__
 
-#include <goabackend/goabackendtypes.h>
+#include <gio/gio.h>
+#include <glib.h>
+#include <glib-object.h>
 
 G_BEGIN_DECLS
 
@@ -35,6 +37,8 @@ G_BEGIN_DECLS
 #define GOA_HTTP_CLIENT(o)          (G_TYPE_CHECK_INSTANCE_CAST ((o), GOA_TYPE_HTTP_CLIENT, GoaHttpClient))
 #define GOA_IS_HTTP_CLIENT(o)       (G_TYPE_CHECK_INSTANCE_TYPE ((o), GOA_TYPE_HTTP_CLIENT))
 
+typedef struct _GoaHttpClient GoaHttpClient;
+
 GType           goa_http_client_get_type           (void) G_GNUC_CONST;
 GoaHttpClient  *goa_http_client_new                (void);
 void            goa_http_client_check              (GoaHttpClient       *client,
@@ -44,7 +48,7 @@ void            goa_http_client_check              (GoaHttpClient       *client,
                                                     gboolean             accept_ssl_errors,
                                                     GCancellable        *cancellable,
                                                     GAsyncReadyCallback  callback,
-                                                    gpointer             gpointer);
+                                                    gpointer             user_data);
 gboolean        goa_http_client_check_finish       (GoaHttpClient       *client,
                                                     GAsyncResult        *res,
                                                     GError             **error);
