@@ -876,116 +876,106 @@ on_web_view_navigation_policy_decision_requested (WebKitWebView             *web
 {
   GoaOAuth2Provider *provider = GOA_OAUTH2_PROVIDER (user_data);
   GoaOAuth2ProviderPrivate *priv = provider->priv;
+  GHashTable *key_value_pairs;
+  SoupMessage *message;
+  SoupURI *uri;
+  const gchar *fragment;
   const gchar *oauth2_error;
+  const gchar *query;
   const gchar *redirect_uri;
   const gchar *requested_uri;
+  gint response_id;
 
   /* TODO: use oauth2_proxy_extract_access_token() */
 
   requested_uri = webkit_network_request_get_uri (request);
   redirect_uri = goa_oauth2_provider_get_redirect_uri (provider);
-  if (g_str_has_prefix (requested_uri, redirect_uri))
+  if (!g_str_has_prefix (requested_uri, redirect_uri))
+    goto default_behaviour;
+
+  message = webkit_network_request_get_message (request);
+  uri = soup_message_get_uri (message);
+  fragment = soup_uri_get_fragment (uri);
+  query = soup_uri_get_query (uri);
+
+  /* Two cases:
+   * 1) we can either have the access_token and other information
+   *    directly in the fragment part of the URI, or
+   * 2) the auth code can be in the query part of the URI, with which
+   *    we'll obtain the token later.
+   */
+  if (fragment != NULL)
     {
-      SoupMessage *message;
-      SoupURI *uri;
+      /* fragment is encoded into a key/value pairs for the token and
+       * expiration values, using the same syntax as a URL query */
+      key_value_pairs = soup_form_decode (fragment);
 
-      message = webkit_network_request_get_message (request);
-      uri = soup_message_get_uri (message);
-
-      /* Two cases:
-       * 1) we can have either the auth code in the query part of the
-       *    URI, with which we'll obtain the token, or
-       * 2) the access_token and other information directly in the
-       *    fragment part of the URI.
+      /* We might use oauth2_proxy_extract_access_token() here but
+       * we can also extract other information.
        */
-      if (soup_uri_get_query (uri) != NULL)
+      priv->access_token = g_strdup (g_hash_table_lookup (key_value_pairs, "access_token"));
+      if (priv->access_token != NULL)
         {
-          GHashTable *key_value_pairs;
+          gchar *expires_in_str = NULL;
 
-          key_value_pairs = soup_form_decode (uri->query);
+          expires_in_str = g_hash_table_lookup (key_value_pairs, "expires_in");
+          /* sometimes "expires_in" appears as "expires" */
+          if (expires_in_str == NULL)
+            expires_in_str = g_hash_table_lookup (key_value_pairs, "expires");
 
-          priv->authorization_code = g_strdup (g_hash_table_lookup (key_value_pairs, "code"));
-          if (priv->authorization_code != NULL)
-            {
-              gtk_dialog_response (priv->dialog, GTK_RESPONSE_OK);
-            }
-          else
-            {
-              oauth2_error = (const gchar *) g_hash_table_lookup (key_value_pairs, "error");
-              if (g_strcmp0 (oauth2_error, GOA_OAUTH2_ACCESS_DENIED) == 0)
-                gtk_dialog_response (priv->dialog, GTK_RESPONSE_CANCEL);
-              else
-                {
-                  g_set_error (&priv->error,
-                               GOA_ERROR,
-                               GOA_ERROR_NOT_AUTHORIZED,
-                               _("Authorization response was \"%s\""),
-                               oauth2_error);
-                  gtk_dialog_response (priv->dialog, GTK_RESPONSE_CLOSE);
-                }
-            }
-          g_hash_table_unref (key_value_pairs);
-          webkit_web_policy_decision_ignore (policy_decision);
+          if (expires_in_str != NULL)
+            priv->access_token_expires_in = atoi (expires_in_str);
+
+          priv->refresh_token = g_strdup (g_hash_table_lookup (key_value_pairs, "refresh_token"));
+
+          response_id = GTK_RESPONSE_OK;
         }
-      else if (soup_uri_get_fragment (uri) != NULL)
-        {
-          GHashTable *key_value_pairs;
-
-          /* fragment is encoded into a key/value pairs for the token and
-           * expiration values, using the same syntax as a URL query */
-          key_value_pairs = soup_form_decode (soup_uri_get_fragment (uri));
-
-          /* We might use oauth2_proxy_extract_access_token() here but
-           * we can also extract other information.
-           */
-          priv->access_token = g_strdup (g_hash_table_lookup (key_value_pairs, "access_token"));
-          if (priv->access_token != NULL)
-            {
-              gchar *expires_in_str = NULL;
-
-              expires_in_str = g_hash_table_lookup (key_value_pairs, "expires_in");
-              /* sometimes "expires_in" appears as "expires" */
-              if (expires_in_str == NULL)
-                expires_in_str = g_hash_table_lookup (key_value_pairs, "expires");
-
-              if (expires_in_str != NULL)
-                 priv->access_token_expires_in = atoi (expires_in_str);
-
-              priv->refresh_token = g_strdup (g_hash_table_lookup (key_value_pairs, "refresh_token"));
-
-              gtk_dialog_response (priv->dialog, GTK_RESPONSE_OK);
-            }
-          else
-            {
-              oauth2_error = (const gchar *) g_hash_table_lookup (key_value_pairs, "error");
-              if (g_strcmp0 (oauth2_error, GOA_OAUTH2_ACCESS_DENIED) == 0)
-                gtk_dialog_response (priv->dialog, GTK_RESPONSE_CANCEL);
-              else
-                {
-                  g_set_error (&priv->error,
-                               GOA_ERROR,
-                               GOA_ERROR_NOT_AUTHORIZED,
-                               _("Authorization response was \"%s\""),
-                               oauth2_error);
-                  gtk_dialog_response (priv->dialog, GTK_RESPONSE_CLOSE);
-                }
-            }
-          g_hash_table_unref (key_value_pairs);
-          webkit_web_policy_decision_ignore (policy_decision);
-        }
-      else
-        {
-          /* this actually means that something unexpected happened, either we
-           * did something wrong or the provider's flow changed */
-          goa_debug ("URI format not recognized, DEFAULT BEHAVIOUR");
-          return FALSE;
-        }
-      return TRUE; /* ignore the request */
+      g_hash_table_unref (key_value_pairs);
     }
+
+  if (priv->access_token != NULL)
+    goto ignore_request;
+
+  if (query != NULL)
+    {
+      key_value_pairs = soup_form_decode (query);
+
+      priv->authorization_code = g_strdup (g_hash_table_lookup (key_value_pairs, "code"));
+      if (priv->authorization_code != NULL)
+        response_id = GTK_RESPONSE_OK;
+
+      g_hash_table_unref (key_value_pairs);
+    }
+
+  if (priv->authorization_code != NULL)
+    goto ignore_request;
+
+  /* In case we don't find the access_token or auth code, then look
+   * for the error in the query part of the URI.
+   */
+  key_value_pairs = soup_form_decode (query);
+  oauth2_error = (const gchar *) g_hash_table_lookup (key_value_pairs, "error");
+  if (g_strcmp0 (oauth2_error, GOA_OAUTH2_ACCESS_DENIED) == 0)
+    response_id = GTK_RESPONSE_CANCEL;
   else
     {
-      return FALSE; /* make default behavior apply */
+      g_set_error (&priv->error,
+                   GOA_ERROR,
+                   GOA_ERROR_NOT_AUTHORIZED,
+                   _("Authorization response was \"%s\""),
+                   oauth2_error);
+      response_id = GTK_RESPONSE_CLOSE;
     }
+  g_hash_table_unref (key_value_pairs);
+  goto ignore_request;
+
+ ignore_request:
+  gtk_dialog_response (priv->dialog, response_id);
+  webkit_web_policy_decision_ignore (policy_decision);
+  return TRUE;
+
+ default_behaviour:
+  return FALSE;
 }
 
 static void
