@@ -36,6 +36,13 @@ G_BEGIN_DECLS
  */
 #define GOA_PROVIDER_EXTENSION_POINT_NAME "goa-backend-provider"
 
+/**
+ * GOA_PROVIDER_FACTORY_EXTENSION_POINT_NAME:
+ *
+ * Extension point for #GoaProviderFactory implementations.
+ */
+#define GOA_PROVIDER_FACTORY_EXTENSION_POINT_NAME "goa-backend-provider-factory"
+
 void goa_provider_ensure_extension_points_registered (void);
 
 G_END_DECLS
