@@ -53,6 +53,8 @@ GType                  goa_web_view_get_type               (void) G_GNUC_CONST;
 GtkWidget             *goa_web_view_new                    (void);
 GtkWidget             *goa_web_view_get_view               (GoaWebView *self);
 void                   goa_web_view_fake_mobile            (GoaWebView *self);
+void                   goa_web_view_add_cookies            (GoaWebView *self,
+                                                            GSList     *cookies);
 
 G_END_DECLS
 
